library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity hall_sensor_sim is
	port (CLOCK_50 : in std_logic;
			BUTTON   : in std_logic_vector(2 downto 0);
			SW       : in std_logic_vector(9 downto 0);
			LEDG     : out std_logic_vector(9 downto 0);
			HEX3_D   : out std_logic_vector(6 downto 0);
			HEX2_D   : out std_logic_vector(6 downto 0);
			HEX1_D   : out std_logic_vector(6 downto 0);
			HEX0_D   : out std_logic_vector(6 downto 0);
			HEX0_DP  : out std_logic;
			HEX1_DP  : out std_logic;
			HEX2_DP  : out std_logic;
			HEX3_DP  : out std_logic;
			-- Hall Sensor Output, mapped to extension header
			HALL_SEN : out std_logic;
			SYNC_OUT : out std_logic
		);
end entity hall_sensor_sim;

architecture rtl of hall_sensor_sim is
signal pulse_counter : integer range 0 to 150000000;
signal rep_counter : integer range 0 to 9;
signal clk, areset : std_logic;
signal hall_sensor : std_logic;
signal speed : std_logic_vector(1 downto 0);
signal sync_output : std_logic;

type state_type is (rest, down_edge, down_edge2, short_wait, up_edge, up_edge2, long_wait);
signal state : state_type;
begin

	clk <= CLOCK_50;
	areset <= not BUTTON(0);
	HALL_SEN <= hall_sensor;
	LEDG(0) <= hall_sensor;
	SYNC_OUT <= sync_output;
	
	process (clk, areset) is
	begin

		if areset = '1' then
			state <= rest;
			hall_sensor <= '0';
			sync_output <= '0';
			pulse_counter <= 0;
			rep_counter <= 0;
			speed <= "00";
		elsif rising_edge(clk) then
			case state is
				when rest =>
					hall_sensor <= '1';
					sync_output <= '1';
					pulse_counter <= 1999;
					rep_counter <= 2;
					state <= down_edge;
					-- latch speed
					speed <= SW(1 downto 0);
				when down_edge =>
					sync_output <= '0';
					hall_sensor <= '0';
					if pulse_counter = 0 then
						pulse_counter <= 1999;
						state <= down_edge2;
					else
						pulse_counter <= pulse_counter - 1;
					end if;
				when down_edge2 =>
					hall_sensor <= '1';
					sync_output <= '0';
					if pulse_counter = 0 then
						if rep_counter = 0 then
							case speed is
								-- 20 rpm
								when "00" => pulse_counter <= 6000000-640;
								-- 85,235 rpm
								when "01" => pulse_counter <= 1407872-640;
								-- 120 rpm
								when "10" => pulse_counter <= 1000000-640;
								-- 200 rpm
								when "11" => pulse_counter <=  600000-640;
								when others => null;
							end case;
							state <= short_wait;
						else
							rep_counter <= rep_counter - 1;
							pulse_counter <= 1999;
							state <= down_edge;
						end if;
					else
						pulse_counter <= pulse_counter - 1;
					end if;
				when short_wait =>
					hall_sensor <= '0';
					sync_output <= '0';
					if pulse_counter = 0 then
						rep_counter <= 1;
						pulse_counter <= 999;
						state <= up_edge;
					else
						pulse_counter <= pulse_counter - 1;
					end if;
					
				when up_edge =>
					hall_sensor <= '1';
					sync_output <= '1';
					if pulse_counter = 0 then
						pulse_counter <= 999;
						state <= up_edge2;
					else
						pulse_counter <= pulse_counter - 1;
					end if;
				when up_edge2 =>
					hall_sensor <= '0';
					sync_output <= '1';
					if pulse_counter = 0 then
						if rep_counter = 0 then
							case speed is
								-- 20 rpm
								when "00" => pulse_counter <= 144000000-15360-3;
								-- 85,235 rpm -- fine tune
								when "01" => pulse_counter <=  33788936-15360-3+1;
								-- 120 rpm
								when "10" => pulse_counter <=  24000000-15360-3;
								-- 200 rpm
								when "11" => pulse_counter <=  14400000-15360-3;
								when others => null;
							end case;
							state <= long_wait;
						else
							rep_counter <= rep_counter - 1;
							pulse_counter <= 999;
							state <= up_edge;
						end if;
					else
						pulse_counter <= pulse_counter - 1;
					end if;
				when long_wait =>
					hall_sensor <= '1';
					sync_output <= '1';
					if pulse_counter = 0 then
						pulse_counter <= 1999;
						state <= rest;
					else
						pulse_counter <= pulse_counter - 1;
					end if;
					
				when others =>
					null;
			end case;
			
		end if;

	end process;
	
	-- The unused outputs...
	HEX3_D <= (others => '1');
	HEX2_D <= (others => '1');
	HEX1_D <= (others => '1');
	HEX0_D <= (others => '1');
	
	HEX3_DP <= '1';
	HEX2_DP <= '1';
	HEX1_DP <= '1';
	HEX0_DP <= '1';
	
	LEDG(9 downto 1) <= (others => '0');
	
end architecture rtl;

